#!/usr/local/bin/perl5.6.0 -w --  # -*-Perl-*-

# $Header: /nfs/pathsoft/repository/psu/genexec/perl/src/align_intron.pl,v 1.3 2002/04/25 12:43:21 kmr Exp $

# purpose: Take the output of intron_extractor.pl and align the output at the
# purpose: branch sites

use strict;

my @branch_patterns;

if (@ARGV && $ARGV[0] =~ /^-h/) {
  die <<EOF;
$0 output_of_intron_extractor.pl
This script takes the output of intron_extractor.pl and align the output at the
branch sites
EOF
}

if (@ARGV && $ARGV[0] eq "-simple") {
  shift;
  @branch_patterns = ('ytray');
} else {
  if (@ARGV && $ARGV[0] eq "-celegans") {
    shift;
    @branch_patterns = ('ytray',
                        'cttac|ctaaa|ataac',
                        'ctcac|ataat|ttaaa|ataaa|ttgat|ataac|gtaac');
  } else {
    @branch_patterns = ('ytray|cttac|ctaaa|ataac|ctcac|gtaac');
  }
}

# now substitute a or g for r and c or t for y
for my $branch_pattern (@branch_patterns) {
  $branch_pattern =~ s/r/[ag]/g;
  $branch_pattern =~ s/y/[ct]/g;
}

open NO_ALIGN, ">non_aligned";

my $gene_name;
my $intron_length;

READ_LOOP: while (<>) {
  if (/^>/) {
    if (m/^>([^\s]+)\s+(\d+)/) {
      $gene_name = $1;
      $intron_length = $2;
    } else {
      warn "can't fathom this line: $_"
    }
  } else {
    $_ = lc $_;

    printf ("%-15s %5d ", $gene_name, $intron_length);

    # match the patterns, align it and turn it to uppercase
    for my $branch_pattern (@branch_patterns) {
      if (m/^((.?.?......\.\.\..*)($branch_pattern)(.*......$))/i) {
        my $all = $1;
        my $before = $2;
        my $pattern = $3;
        my $after = $4;

        my $indent_level = 48 - length $before;

        my $first_bit = $before;     # the bit before the branch

        my $second_bit =
          "\U$pattern\E$after"; # the bit from the branch onwards

        # turn all occurrences of the patterns before the branch to uppercase
        for my $branch_pattern (@branch_patterns,@branch_patterns) {
          $first_bit =~ s/($branch_pattern)/\U$1\E/ig;
        }

        # if the last two bases aren't ag then highlight by turning to
        # uppercase
        for my $branch_pattern (@branch_patterns) {
          if ($second_bit !~ /ag$/i) {
            $second_bit =~ s/(..)$/\U$1\E/g;
          }
        }

        # we can align it:
        print  " " x $indent_level, "$first_bit$second_bit\n";

        next READ_LOOP;
      }
    }

    # turn all occurrences of the patterns before the branch to uppercase
    for my $branch_pattern (@branch_patterns) {
      s/($branch_pattern)/\U$1\E/ig;
    }

    # we only get here if alignment was not found
    print;

    printf NO_ALIGN "%-15s %5d ", $gene_name, $intron_length;
    print NO_ALIGN;
  }
}
