#!/usr/local/bin/perl5.6.0 -w

# External Modules

# Constraint variables declaration before using them
use strict;
##########################################

use Data::Dumper;

# Benchmark Module
use Benchmark;
##################################################################

#######
# blast parser
##

 eval "require Bio::PSU::Utils::BlastParsing";
# use Bio::PSU::Utils::BlastParsing;

##########################################
=pod

=item * Configuration loading from the command line

=cut
##########################################

my $t1 = Benchmark->new ();


############################
# Debug boolean
##

my $debug = 0;

my $_help           = "-help";
my $_sequences_url  = "-urlfasta";
my $_droso          = "-droso";
my $_swall          = "-swall";
my $_title          = "-title";

# no blast output per default
my $droso = "-1";
my $swall = "-1";

######################
# HTML Page Title
##
my $title = "Glossina morsitans morsitans ESTs Annotation report";

######################
# Sequence URL
##
my $sequences_url = "ftp://ftp.sanger.ac.uk/pub/pathogens/Glossina/ESTs";

######################
# Mapping Files
##
my $pfam_mapping_file = "/nfs/pathsoft/external/bio-soft/proteinMappingFiles/pfamName2Accession.mapping";
my $ipro_mapping_file = "/nfs/pathsoft/external/bio-soft/proteinMappingFiles/protein2ipr.dat";
my $go_mapping_file   = "/nfs/pathsoft/external/bio-soft/proteinMappingFiles/interpro2go.txt";
my $droso_gene_mapping_file = "/nfs/pathsoft/databases/go/gene-associations/gene_association.fb";

#######################
# OTHER URLs (Swall and Drosophila URLs)
##
my $swall_entry_url = "http://www.expasy.ch/cgi-bin/niceprot.pl?";
my $droso_entry_url = "http://flybase.bio.indiana.edu/.bin/fbidq.html?";

###################
# Maximum of reported hits
##
my $max_hits = 3;

my $i = 0;
foreach my $element (@ARGV) {
  if ($element eq $_help) {
    display_help ();
    exit 1;
  }
  elsif ($element eq $_title) {
    $title = $ARGV[$i+1];
  }
  elsif ($element eq $_sequences_url) {
    $sequences_url = $ARGV[$i+1];
  }
  elsif ($element eq $_droso) {
    $droso = $ARGV[$i+1];
  }
  elsif ($element eq $_swall) {
    $swall = $ARGV[$i+1];
  }
  # if at least one of the options is not recognized, display the help message and exit
  elsif ($element =~ /^-/) {
    print STDERR "option not recognized: $element\n";
    display_help ();
    exit 1;
  }
  $i++;
}

if (($title eq "-1") || ($droso eq "-1") || ($swall eq "-1") || ($sequences_url eq "-1")) {
  print STDERR "option(s) not specified!!\n";
  display_help ();
  exit 1;
}


#############################
# Parsing blastx results
##

print STDERR "parsing droso blast output...\n";

my $blast_parser = Bio::PSU::Utils::BlastParsing->new (
						       blastOutput => $droso,
						       max_hits => $max_hits,
						      );
my %results_blastx_droso = $blast_parser->parse;

if ($debug) {
  print STDERR "#############\nblast results object for droso: " . Dumper (%results_blastx_droso) . "\n#############\n";
}

print STDERR "parsing swall blast output...\n";

$blast_parser->setBlastOutput ($swall);
my %results_blastx_swall = $blast_parser->parse;

###########################
# Generate HTML output
##

my $html_blast_name   = "ESTs.blastx";
generateBlastxHTMLOutput ($html_blast_name, \%results_blastx_droso, \%results_blastx_swall, $droso_gene_mapping_file);

print STDERR "generation done.\n";

my $t2 = Benchmark->new ();

print STDERR "\nTotal : ", timestr (timediff ($t2, $t1)), "\n";

###############
# end
##

############################
# display the help message
##

sub display_help {
  print "blast2html.pl $_help $_droso [blastx results vs Drosophila] $_swall [blastx results vs Swall] $_sequences_url [ESTs file URL] $_title [HTML Page Title]\n\n";
  print "\t$_help a help message\n";
  print "\t$_sequences_url the sequences URL - (default is Glossina ESTs ftp URL)\n";
  print "\t$_title the HTML Page Title\n\n";
  print "\t$_droso the blastx output file or directory Vs Drosophila - (no default)\n";
  print "\t$_swall the blastx output file or directory Vs Swall - (no default)\n\n";
}


####################################
# Generate the HTML pages
# 500 results per HTML page
##

sub generateBlastxHTMLOutput {
  my ($html_blastx_name, $results_blastx_droso_ref, $results_blastx_swall_ref, $droso_gene_mapping_file) = @_;

  my %results_blastx_droso = %{$results_blastx_droso_ref};
  my %results_blastx_swall = %{$results_blastx_swall_ref};

  if ($debug) {
    print STDERR "#############\nblast results object for droso: " . Dumper (%results_blastx_droso) . "\n#############\n";
    print STDERR "#############\nblast results object for swall: " . Dumper (%results_blastx_swall) . "\n#############\n";
  }

  print STDERR "generating blast html output ...\n";

  my @keys_droso   = keys (%results_blastx_droso);
  my @keys_swall   = keys (%results_blastx_swall);
 
  my @keys_only_swall = hit_only_one_db (\@keys_swall, \@keys_droso);

  my @keys_only_droso = hit_only_one_db (\@keys_droso, \@keys_swall);

  print STDOUT "\t* " . @keys_swall  . " sequences hit swall.\n";
  print STDOUT "\t* " . @keys_droso  . " sequences hit the drosophila protein database.\n";
  print STDOUT "\t* " . @keys_only_swall . " sequences hit swall and only swall.\n";
  
  print STDOUT "\t* " . @keys_only_droso . " sequences hit drosophila database and only drosophila.\n";

  my @blastx_keys = @keys_swall;

  @blastx_keys = union (\@blastx_keys, \@keys_droso);
  print STDERR "union of swall and droso keys: " . @blastx_keys . "\n";

  @blastx_keys  = sort_per_key (@blastx_keys);

  if ($debug) {
    print STDERR "keys after sorting: @blastx_keys\n";
  }

  my $color = "silver";
  my $j = 0;
  my $nb_ests_per_page = 500;
  my $nb_pages = int (@blastx_keys/$nb_ests_per_page) + 1;
  my $page = 1;
  my $page_previous = 1;
  my $page_next = 2;

  my $html_file = "$html_blastx_name.html";

  foreach my $key (@blastx_keys) {
    if ($nb_pages > 1) {
      $html_file = "$html_blastx_name.$page.html";
    }

    open HTML_BLASTX, ">> $html_file" or die "can't open file, $html_file\n";

    if (($j%$nb_ests_per_page) == 0) {
      
      print HTML_BLASTX "<html>\n<body>\n<center><h2>$title</h2></center><br>\n";
   
      if ($nb_pages > 1) {
	print HTML_BLASTX "<br><h3><center><a href=\"$html_blastx_name.1.html\">first</a><a href=\"$html_blastx_name.$page_previous.html\">  previous</a><a href=\"$html_blastx_name.$page_next.html\">  next</a><a href=\"$html_blastx_name.$nb_pages.html\">  last</a></center></h3><br>\n";
      }

      print HTML_BLASTX "<table border><tr><th>.</th><th colspan=4>Blastx Vs Swall</th><th colspan=5>Blastx Vs Drosophila</th></tr><tr><th>EST</th><th>Subject ID</th><th>Taxon</th><th>%Identity</th><th>Score</th><th>E-Value</th><th>Subject ID</th><th>%Identity</th><th>Score</th><th>E-Value</th></tr>\n";
    }

    if ($debug) {
      print STDERR "current key: $key\n";
    }

    my $est_acc = $key;

    # "CONTIG10" => "contig10"
    #if ($est_acc =~ /CONTIG/) {
    #  $est_acc = lc ($est_acc);
    #}

    my $i = 0;

    my @results_droso = ();
    my @results_swall = ();
    if (exists $results_blastx_droso{$key}) {
      @results_droso = @{$results_blastx_droso{$key}};
    }
    if (exists $results_blastx_swall{$key}) {
      @results_swall = @{$results_blastx_swall{$key}};
    }
    
    while (($i<@results_droso) || ($i<@results_swall)) {
      
      print HTML_BLASTX "<tr bgcolor=$color>\n";

      ##########
      # swall
      ##

      if ($i<@results_swall) {
	my $hit = $results_swall[$i];
	# print STDERR "new hit...\n";
	my $s_acc   = $hit->getSubjectAccession;
	my $s_desc  = $hit->getSubjectDescription;
	# make it lowercase
	$s_desc = lc ($s_desc);
	my $score   = $hit->getScore;
	my $percent = int ($hit->getPercent);
	my $e_value = $hit->getEValue;

	# get the organism

	# my @args = ("getz \'[SWALL-acc:$s_acc]\' -f \"org\"");
	my $organism = qx(getz '[SWALL-acc:$s_acc]|[SWALL-id:$s_acc]' -f "org");
	if ($debug) {
	  print STDERR "getz organism: $organism from swall accession number, $s_acc\n";
	}
	$organism =~ /OS\s+(.+)\./;
	$organism = $1;
	if ($organism =~ /(.+)\s+\(.+/) {
	  $organism = $1;
	}
	if ($debug) {
	  print STDERR "parsed organism: $organism\n";    
	  print STDERR "score, %, evalue: $score, $percent, $e_value\n";
	}
	
	print HTML_BLASTX "<td><a href=\"$sequences_url/$est_acc.fasta\">$est_acc</a></td><td><a href=\"$swall_entry_url$s_acc\">$s_desc</a></td><td>$organism</td><td>$percent%</td><td>$score</td><td>$e_value</td>";
      }
      else {
	print HTML_BLASTX "<td><a href=\"$sequences_url/$est_acc.fasta\">$est_acc</a></td><td>.</td><td>.</td><td>.</td><td>.</td><td>.</td>";
      }
      
      #########
      # droso
      ##

      if ($i<@results_droso) {
	my $hit = $results_droso[$i];
	# print STDERR "new hit...\n";
	my $s_acc   = $hit->getSubjectAccession;
	my $s_desc  = $hit->getSubjectDescription;
	my $s_name  = mapping_droso_gene_accession_name ($droso_gene_mapping_file, $s_acc);
	my $score   = $hit->getScore;
	my $percent = int ($hit->getPercent);
	my $e_value = $hit->getEValue;
	
	# parsing the description to make it less messy !!
	
	if ($s_desc =~ /(.+)\s*mol_weight.+/) {
	  $s_desc = $1;
	  if ($debug) {
	    print STDERR "s_desc after parsing: $s_desc, gene name: $s_name\n";
	  }
	}

	# the description contains a list of go terms - if no go terms have been assigned, then display the gene name

	if ($s_desc =~ /GO\:\[\]/) {
	  if ($s_name ne "") {
	    $s_desc = $s_name;
	  }
	  else {
	    $s_desc = $s_acc;
	  }
	  if ($debug) {
	    print STDERR "s_desc has changed: $s_desc\n";
	  }
	}

	if ($debug) {
	  print STDERR "score, %, evalue: $score, $percent, $e_value\n";
	}

	print HTML_BLASTX "<td><a href=\"$droso_entry_url$s_acc\">$s_desc</a></td><td>$percent%</td><td>$score</td><td>$e_value</td>";
      }
      else {
	print HTML_BLASTX "<td>.</td><td>.</td><td>.</td><td>.</td>";
      }

      $i++;
      print HTML_BLASTX "</tr>\n";
    }
    if ($color =~ /silver/ ) {
      $color = "white";
    }
    else {
      $color = "silver";
    }
    $j ++;

    if (($j%$nb_ests_per_page) == 0) {

      # add some test to avoid the next on the last one and the previous on the first one !!
      
      print HTML_BLASTX "</table>\n<br><h3><center><a href=\"$html_blastx_name.1.html\">first</a><a href=\"$html_blastx_name.$page_previous.html\">  previous</a><a href=\"$html_blastx_name.$page_next.html\">  next</a><a href=\"$html_blastx_name.$nb_pages.html\">  last</a></center><br></h3>\n";
      print HTML_BLASTX "</body>\n</html>";
      
      $page_previous = $page;
      $page ++;
      $page_next     = $page + 1;
    }
    close HTML_BLASTX;
  } # end foreach key

  if (($j%$nb_ests_per_page) != 0) {
    open HTML_BLASTX, ">> $html_file" or die "can't open file, $html_file\n";
    
    # add some test to avoid the next on the last one and the previous on the first one !!
    
    print HTML_BLASTX "</table>\n";

    if ($nb_pages > 1) {
      print HTML_BLASTX "<br><h3><center><a href=\"$html_blastx_name.1.html\">first</a><a href=\"$html_blastx_name.$page_previous.html\">  previous</a><a href=\"$html_blastx_name.$page_next.html\">  next</a><a href=\"$html_blastx_name.$nb_pages.html\">  last</a></center><br></h3>\n";
    }

    print HTML_BLASTX "</body>\n</html>";
    close HTML_BLASTX;
  } 
}


sub union {
  my ($all_keys_ref, $keys_droso_ref) = @_;
  my @all_keys   = @$all_keys_ref;
  my @keys_droso = @$keys_droso_ref;

  foreach my $key (@keys_droso) {
    if (! is_in ($key, @all_keys)) {
      if ($debug) {
	print STDERR "add droso key.\n";
      }
      push (@all_keys, $key);
    }
  }
  return @all_keys;
}

sub hit_only_one_db {
  my ($keys_swall_ref, $keys_droso_ref) = @_;
  my @keys_swall = @$keys_swall_ref;
  my @keys_droso = @$keys_droso_ref;
  my @keys_only_swall = ();

  my $file = "sequences.swall.droso";
  open FILE, ">$file" or die "can't open file, $file!\n";

  foreach my $key_swall (@keys_swall) {
    if (! is_in ($key_swall, @keys_droso)) {
      if ($debug) {
	print STDERR "add swall key, not hiting droso...\n";
      }
      push (@keys_only_swall, $key_swall);
    }
    else {
      # in both database, store the EST in a file
      print FILE "$key_swall\n";
    }
  }

  close FILE;

  return @keys_only_swall;
}

sub is_in {
  my ($key, @all_keys) = @_;

  foreach my $key_tmp (@all_keys) {
    if ($key =~ /^$key_tmp$/) {
      # print STDERR "$key matches $key_tmp ? yes!\n";
      return 1;
    }
  }
  return 0;
}


sub mapping_droso_gene_accession_name {
  my ($droso_gene_mapping_file, $s_acc) = @_;
  my $gene_name = "";
  
  open MAPPING, "< $droso_gene_mapping_file" or die "can't open file, $droso_gene_mapping_file\n";

  while (<MAPPING>) {
    my $line = $_;
    if ($line =~ /.+$s_acc.+/) {
      if ($debug) {
	print STDERR "line: $line\n";
      }
      $line =~ /.+$s_acc\s+(\S+)\s+.+/;
      $gene_name = $1;
      last;
    }
  }

  close MAPPING;

  if ($debug) {
    print STDERR "gene acc, gene name: $s_acc, $gene_name\n";
  }
  
  return $gene_name;
}

sub sort_per_key {
  my (@keys) = @_;
  my @singlet_keys = ();
  my @indexes = ();

  foreach my $key (@keys) {
    if ($debug) {
      print STDERR "key to sort: $key\n";
    }
    # it is a contig
    if ($key =~ /CONTIG(\d+)/i) {
      my $index = $1;
      push (@indexes, $index);
    }
    # it is a singlet
    else {
      push (@singlet_keys, $key);
    }
  }

  if ($debug) {
    print STDERR "indexes before sorting: @indexes\n";
  }

  @indexes = sort { $a <=> $b } @indexes;

  if ($debug) {
    print STDERR "indexes after sorting: @indexes\n";
  }

  my $i = 0;
  while ($i<@indexes) {
    my $index = $indexes[$i];
    $indexes[$i] = "CONTIG$index";
    $i++;
  }
  @singlet_keys = sort (@singlet_keys);
  push (@indexes, @singlet_keys);
  return @indexes;
}
