#!/usr/local/bin/perl -w

use strict;

# $Header: /nfs/pathsoft/repository/psu/genexec/perl/src/find_feature.pl,v 1.3 1999/11/16 14:15:37 kmr Exp $

# purpose: This script will grep a feature table file for a regular expression
# purpose: given on the command line.  If a match is found the whole feature
# purpose: is printed.

sub usage
{
  die <<EOF;
usage: $0 -h (for help)
       $0 [-i] [-v] [-l] [-L] search_regular_expression tabfile ..

flags:  -i will cause the search to ignore case
        -v will invert the search - only non-matching features will be
           printed
        -l lists only the name of each file containing matched features
        -L prints the name of each file containing the matched feature at the
           start of each output line

This script will grep a feature table file for a regular expression
given on the command line.  If a match is found the whole feature
is printed.
EOF
}

my $seen_non_feature_line = 0;

my $ignore_case = 0;
my $invert = 0;
my $file_names_only = 0;
my $file_name_lines = 0;

while (@ARGV && $ARGV[0] =~ /^-(.)$/) {
  usage if ($1 eq 'h');

  $ignore_case = 1,     shift, next if ($1 eq 'i');
  $invert = 1,          shift, next if ($1 eq 'v');
  $file_names_only = 1, shift, next if ($1 eq 'l');
  $file_name_lines = 1, shift, next if ($1 eq 'L');

  usage;
}

if (!@ARGV) {
  usage;
}

my $search_re = $ARGV[0];

shift;

sub print_feature_helper
{
  my $feature = $_[0];
  
  if ($file_name_lines) {
    $feature =~ s/^/$ARGV:/mg;
  }
  
  print $feature;
}

sub print_feature
{
  my ($feature, $feature_matches) = @_;
  
  if ($invert) {
    if (!$feature_matches) {
      if ($file_names_only) {
        print "$ARGV\n";
        close ARGV;
      } else {
        print_feature_helper $feature;          
      }
    }
  } else {
    if ($feature_matches) {
      if ($file_names_only) {
        print "$ARGV\n";
        close ARGV;
      } else {
        print_feature_helper $feature;          
      }
    }
  }
}

sub check_feature
{
  my $feature = $_[0];
  
  if ($ignore_case) {
    if ($feature =~ m/$search_re/i) {
      print_feature $feature, 1;
      1;
    } else {
      print_feature $feature, 0;
      0;
    }
  } else {
    if ($feature =~ m/$search_re/) {
      print_feature $feature, 1;
      1;
    } else {
      print_feature $feature, 0;
      0;
    }
  }
}

# set to 1 if a match is found
my $found_match = 0;

my $feature_so_far = "";

while (<>) {
  if (! m/FT   (...............) (.*)/) {

    if (!$seen_non_feature_line) {
      print STDERR "non-feature line at line $. of input:\n";
      print STDERR;
      $seen_non_feature_line = 1;
    }
    next;
  }

  my $key_bit = $1;

  $key_bit =~ s/ //g;

  my $rest_of_line = $2;

  if (length $key_bit > 0) {
    # first line of a feature 

    my $feature_matches = check_feature ($feature_so_far);

    if ($feature_matches) {
      $found_match = 1;
    }

    $feature_so_far = "";
  }

  $feature_so_far .= $_;
}

check_feature ($feature_so_far);

if ($found_match) {
  exit 0
} else {
  exit 1
}
