#!/usr/local/bin/perl -w

use strict;

BEGIN {
  push (@INC, "/nfs/disk92/PerlSource/Bioperl/Releases/bioperl-0.05.1/");
  push (@INC, "/nfs/disk222/yeastpub/scripts/perl/LocalModules/");
}

use Bio::Seq;
use Bio::SeqIO;

if (@ARGV != 2) {
  die "usage: $0 sequence1 sequence2\n";
}

my ($seq1, $seq2) = @ARGV;

open MUMMER1, "unique-match $seq1 $seq2|" or
  die "couldn't start unique-match\n";

while (<MUMMER1>) {
  if (1../Start1/) {
    next
  }

  chomp;
  s/\s+/ /g;
  s/^\s+//;

  print "$_ +\n";

  if (eof) {
    last
  }
}

my $seq2_rev = $seq2 . ".rev";


my $wait_val = system "revseq $seq2 -outseq $seq2_rev";

my $revseq_retval = (int($wait_val) / 256);

if ($revseq_retval > 0) {
  die "revseq failed with code $wait_val $revseq_retval\n";
}

open MUMMER2, "unique-match $seq1 $seq2_rev|" or
  die "couldn't start unique-match\n";

my $seqio = Bio::SeqIO->new (-file => "$seq2", '-format' => 'Fasta');

my $seq_len;

{
  my $seq = $seqio->next_seq ();

  $seq_len = $seq->seq_len ();

  print STDERR "sequence length: $seq_len\n";;
}

while (<MUMMER2>) {
  if (1../Start1/) {
    next
  }

  if (/(\d+)\s+(\d+)\s+(\d+)/) {
    print "$1 ", $seq_len - ($2 + $3 - 1), " $3 -\n";
  } else {
    die
  }
}
